library IEEE;
use IEEE.std_logic_1164.all;
use work.codec.all;

entity loopback is
	port
	(
		clk: in std_logic;
		rst: in std_logic;
		mclk: out std_logic;
		lrck: out std_logic;
		sclk: out std_logic;
		sdout: in std_logic;
		sdin: out std_logic;
		s: out std_logic_vector(1 downto 0)
	);
end loopback;

architecture loopback_arch of loopback is
signal lrsel,rd,wr: std_logic;
signal left_channel,right_channel: std_logic_vector(19 downto 0);
signal ldac_in_rdy,rdac_in_rdy: std_logic;
signal ladc_out_rdy,radc_out_rdy: std_logic;
begin
	u0: codec_intfc 
				generic map
				(
					XSTEND_V1_2=>false, -- false for XStend V1.3 board, true for XStend V1.2 board
					ADC_WIDTH=>20,
					DAC_WIDTH=>20
				)
				port map
				(
					clk=>clk,
					reset=>rst,
					mclk=>mclk,
					sclk=>sclk,
					lrck=>lrck,
					sdout=>sdout,
					sdin=>sdin,
					lrsel=>lrsel,
					rd=>rd,
					wr=>wr,
					ladc_out=>left_channel,
					ldac_in=>left_channel,
					radc_out=>right_channel,
					rdac_in=>right_channel,
					ladc_out_rdy=>ladc_out_rdy,
					radc_out_rdy=>radc_out_rdy,
					ldac_in_rdy=>ldac_in_rdy,
					rdac_in_rdy=>rdac_in_rdy,
					dac_underrun=>s(0),
					adc_overrun=>s(1)
				);

	process(ldac_in_rdy,ladc_out_rdy,rdac_in_rdy,radc_out_rdy)
	begin
		if(ladc_out_rdy=YES and ldac_in_rdy=YES) then
			lrsel<=LEFT;
			rd<=YES;
			wr<=YES;
		elsif(radc_out_rdy=YES and rdac_in_rdy=YES) then
			lrsel<=RIGHT;
			rd<=YES;
			wr<=YES;
		else
			lrsel<=LEFT;
			rd<=NO;
			wr<=NO;	
		end if;
	end process;
end loopback_arch;

